dnl ---------------------------------------------------------------------------
dnl
dnl  particles.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(particles.c,
                the soft particles which represent polymers.)

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "mt.h"
#include "particles.h"

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static soft_particles *allocate_soft_particles@D@(polymer_blend *blend);
static void free_soft_particles@D@(soft_particles *particles);
static void set_particle_positions@D@_random(soft_particles@D@ *particles,
                                             polymer_blend *blend,
                                             box_geometry *geometry);
static void set_particle_positions@D@_into_box(soft_particles@D@ *particles,
                                               vector@D@ **r,
                                               box_geometry *geometry);'`')

/*---------------------------------------------------------------------------*/
soft_particles *allocate_soft_particles(polymer_blend *blend,
                                        box_geometry *geometry)
{
    soft_particles *particles;

    SWITCH_FUNCTION(geometry->dimension,
                    particles = allocate_soft_particles@D@(blend);,
                    particles = NULL;)

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_soft_particles(soft_particles *particles)
{
    SWITCH_FUNCTION(particles->dimension,
                    free_soft_particles@D@(particles);)
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_random(soft_particles *particles,
                                   polymer_blend *blend,
                                   box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    set_particle_positions@D@_random(&(particles->particles@D@),blend,geometry);)
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_into_box(soft_particles *particles,
                                     box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    set_particle_positions@D@_into_box(&(particles->particles@D@),particles->particles@D@.r,geometry);)
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_star_into_box(soft_particles *particles,
                                          box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    set_particle_positions@D@_into_box(&(particles->particles@D@),particles->particles@D@.r_star,geometry);)
}

/*---------------------------------------------------------------------------*/
int total_number_of_soft_particles(soft_particles *particles)
{
    int m_tot;

    SWITCH_FUNCTION(particles->dimension,
                    m_tot = total_number_of_soft_particles@D@(&(particles->particles@D@));,
                    m_tot = 0;)

    return m_tot;
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
soft_particles *allocate_soft_particles@D@(polymer_blend *blend)
{
    int i,m_tot;
    soft_particles *particles;
    soft_particles@D@ *particles@D@;

    particles = xmalloc(sizeof(soft_particles));
    particles->dimension = @D@;

    particles@D@ = &(particles->particles@D@);
    particles@D@->n = blend->n;
    particles@D@->m = xmalloc(sizeof(int) * blend->n);
    particles@D@->r = xmalloc(sizeof(vector@D@ *) * blend->n);
    particles@D@->f = xmalloc(sizeof(vector@D@ *) * blend->n);
    particles@D@->w = xmalloc(sizeof(vector@D@ *) * blend->n);
    particles@D@->r_star = xmalloc(sizeof(vector@D@ *) * blend->n);
    particles@D@->f_star = xmalloc(sizeof(vector@D@ *) * blend->n);
    particles@D@->sigma = xmalloc(sizeof(symmetric_tensor@D@ *) * blend->n);

    m_tot = 0;
    for(i = 0;i < particles@D@->n;i++)
    {
        particles@D@->m[i] = blend->m[i];
        m_tot += particles@D@->m[i];
    }

    particles@D@->r[0] = xmalloc(sizeof(vector@D@) * m_tot);
    particles@D@->f[0] = xmalloc(sizeof(vector@D@) * m_tot);
    particles@D@->w[0] = xmalloc(sizeof(vector@D@) * m_tot);
    particles@D@->r_star[0] = xmalloc(sizeof(vector@D@) * m_tot);
    particles@D@->f_star[0] = xmalloc(sizeof(vector@D@) * m_tot);
    particles@D@->sigma[0] = xmalloc(sizeof(symmetric_tensor@D@) * m_tot);

    for(i = 0;i < (particles@D@->n - 1);i++)
    {
        particles@D@->r[i + 1] = particles@D@->r[i] + particles@D@->m[i];
        particles@D@->f[i + 1] = particles@D@->f[i] + particles@D@->m[i];
        particles@D@->w[i + 1] = particles@D@->w[i] + particles@D@->m[i];
        particles@D@->r_star[i + 1] = particles@D@->r_star[i] + particles@D@->m[i];
        particles@D@->f_star[i + 1] = particles@D@->f_star[i] + particles@D@->m[i];
        particles@D@->sigma[i + 1] = particles@D@->sigma[i] + particles@D@->m[i];
    }

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_soft_particles@D@(soft_particles *particles)
{
    soft_particles@D@ *particles@D@;
    if(particles == NULL)
        return;

    particles@D@ = &(particles->particles@D@);

    free(particles@D@->r[0]);
    free(particles@D@->f[0]);
    free(particles@D@->w[0]);
    free(particles@D@->r_star[0]);
    free(particles@D@->f_star[0]);
    free(particles@D@->sigma[0]);
    free(particles@D@->r);
    free(particles@D@->f);
    free(particles@D@->w);
    free(particles@D@->r_star);
    free(particles@D@->f_star);
    free(particles@D@->sigma);
    free(particles@D@->m);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions@D@_random(soft_particles@D@ *particles,
                                      polymer_blend *blend,
                                      box_geometry *geometry)
{
    int i,j;

    for(i = 0;i < particles->n;i++)
    {
        if(blend->next[i] < 0)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                VECTOR@D@(particles->r[i][j][@@] = geometry->l[@@] * mtrand_real2();)
            }
        }
    }

    for(i = 0;i < particles->n;i++)
    {
        if(blend->next[i] >= 0)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                VECTOR@D@(particles->r[i][j][@@] = particles->r[blend->next[i]][j][@@];)
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions@D@_into_box(soft_particles@D@ *particles,
                                        vector@D@ **r,
                                        box_geometry *geometry)
{
    int i,j;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                VECTOR@D@(while(r[i][j][@@] < 0.0)
                              r[i][j][@@] += geometry->l[@@];
                          while(r[i][j][@@] >= geometry->l[@@])
                              r[i][j][@@] -= geometry->l[@@];)
            }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                while(r[i][j][1] < 0.0)
                {
                    r[i][j][1] += geometry->l[1];
                    r[i][j][0] += geometry->delta_l;
                }
                while(r[i][j][1] >= geometry->l[1])
                {
                    r[i][j][1] -= geometry->l[1];
                    r[i][j][0] -= geometry->delta_l;
                }
                while(r[i][j][0] < 0.0)
                    r[i][j][0] += geometry->l[0];
                while(r[i][j][0] >= geometry->l[0])
                    r[i][j][0] -= geometry->l[0];
                ifelse(@D@,3,
                       while(r[i][j][2] < 0.0)
                           r[i][j][2] += geometry->l[2];
                       while(r[i][j][2] >= geometry->l[2])
                           r[i][j][2] -= geometry->l[2];)
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
int total_number_of_soft_particles@D@(soft_particles@D@ *particles)
{
    int i;
    int m_tot;

    m_tot = 0;
    for(i = 0;i < particles->n;i++)
    {
        m_tot += particles->m[i];
    }

    return m_tot;
}'`')

/*---------------------------------------------------------------------------*/
