/*---------------------------------------------------------------------------*/

/*!  \file  textread.c
     \brief  read the field data from an ascii text file.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from textread.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "particles.h"
#include "textread.h"
#include "zget.h"

#define STRING_BUFFERSIZE 0x100

/*---------------------------------------------------------------------------*/
static void read_positions2_text(char *filename,soft_particles2 *particles);

static void read_positions3_text(char *filename,soft_particles3 *particles);

/*---------------------------------------------------------------------------*/
void read_positions_text(char *filename,soft_particles *particles)
{
        switch(particles->dimension)
    {
        case 2:
            read_positions2_text(filename,&(particles->particles2));
            break;

        case 3:
            read_positions3_text(filename,&(particles->particles3));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void read_positions2_text(char *filename,soft_particles2 *particles)
{
    gzFile *fp;
    int i,j;
    static char buf[STRING_BUFFERSIZE];
    int d,n,m;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"rb");

        /*  check the header  */
    gzgetstring(fp,buf,STRING_BUFFERSIZE);
    if(strcmp(buf,"#") != 0)
    {
        goto error;
    }

    gzgetint(fp,&d);
    if(d != 2)
    {
        goto error;
    }

    gzgetint(fp,&n);
    if(n != particles->n)
    {
        goto error;
    }

    gzgetstring(fp,buf,STRING_BUFFERSIZE);
    if(strcmp(buf,"#") != 0)
    {
        goto error;
    }

    for(i = 0;i < particles->n;i++)
    {
        gzgetint(fp,&m);
        if(m != particles->m[i])
        {
            goto error;
        }
    }


    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            gzgetdouble(fp,&(particles->r[i][j][0]));
gzgetdouble(fp,&(particles->r[i][j][1]));
        }
    }

    gzclose(fp);
    return;


  error:
    gzclose(fp);

    fprintf(stderr,"cannot read data from a file `%s' (incorrect format or size mismatch)\n",filename);
    exit(EXIT_FAILURE);
}

/*---------------------------------------------------------------------------*/
void read_positions3_text(char *filename,soft_particles3 *particles)
{
    gzFile *fp;
    int i,j;
    static char buf[STRING_BUFFERSIZE];
    int d,n,m;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"rb");

        /*  check the header  */
    gzgetstring(fp,buf,STRING_BUFFERSIZE);
    if(strcmp(buf,"#") != 0)
    {
        goto error;
    }

    gzgetint(fp,&d);
    if(d != 3)
    {
        goto error;
    }

    gzgetint(fp,&n);
    if(n != particles->n)
    {
        goto error;
    }

    gzgetstring(fp,buf,STRING_BUFFERSIZE);
    if(strcmp(buf,"#") != 0)
    {
        goto error;
    }

    for(i = 0;i < particles->n;i++)
    {
        gzgetint(fp,&m);
        if(m != particles->m[i])
        {
            goto error;
        }
    }


    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            gzgetdouble(fp,&(particles->r[i][j][0]));
gzgetdouble(fp,&(particles->r[i][j][1]));
gzgetdouble(fp,&(particles->r[i][j][2]));
        }
    }

    gzclose(fp);
    return;


  error:
    gzclose(fp);

    fprintf(stderr,"cannot read data from a file `%s' (incorrect format or size mismatch)\n",filename);
    exit(EXIT_FAILURE);
}

/*---------------------------------------------------------------------------*/
