dnl ---------------------------------------------------------------------------
dnl
dnl  textwrite.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2008 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(textwrite.c,
                output the field data as ascii text format.)

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "particles.h"
#include "textwrite.h"

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static void write_positions@D@_text(char *filename,soft_particles@D@ *particles);
static void write_forces@D@_text(char *filename,soft_particles@D@ *particles);
static void write_stresses@D@_text(char *filename,soft_particles@D@ *particles,
                                   box_geometry *geometry);
static void calculate_system_stress_tensor@D@(soft_particles@D@ *particles,
                                              box_geometry *geometry,
                                              symmetric_tensor@D@ sigma);'`')

/*---------------------------------------------------------------------------*/
void write_positions_text(char *filename,soft_particles *particles)
{
    SWITCH_FUNCTION(particles->dimension,
                    write_positions@D@_text(filename,&(particles->particles@D@));)
}

/*---------------------------------------------------------------------------*/
void write_forces_text(char *filename,soft_particles *particles)
{
    SWITCH_FUNCTION(particles->dimension,
                    write_forces@D@_text(filename,&(particles->particles@D@));)
}

/*---------------------------------------------------------------------------*/
void write_stresses_text(char *filename,soft_particles *particles,
                         box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    write_stresses@D@_text(filename,&(particles->particles@D@),
                                           geometry);)
}

/*---------------------------------------------------------------------------*/
void write_box_geometry_text(char *filename,box_geometry *geometry)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = fopen(filename,"wb");

    switch(geometry->dimension)
    {
        case 2:
            fprintf(fp,"2\n");
            fprintf(fp,"%f %f\n",geometry->l[0],geometry->l[1]);
            fprintf(fp,"%f\n",geometry->delta_l);
            break;

        case 3:
            fprintf(fp,"3\n");
            fprintf(fp,"%f %f %f\n",geometry->l[0],geometry->l[1],geometry->l[2]);
            fprintf(fp,"%f\n",geometry->delta_l);
            break;

        default:
            break;
    }

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void write_positions@D@_text(char *filename,soft_particles@D@ *particles)
{
    gzFile *fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# @D@ %d\n# ",particles->n);
    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%d ",particles->m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            ifelse(@D@,2,
                   gzprintf(fp,"%f %f\n",particles->r[i][j][0],
                            particles->r[i][j][1]);,
                   @D@,3,
                   gzprintf(fp,"%f %f %f\n",particles->r[i][j][0],
                            particles->r[i][j][1],particles->r[i][j][2]);)
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_forces@D@_text(char *filename,soft_particles@D@ *particles)
{
    gzFile *fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# @D@ %d\n# ",particles->n);
    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%d ",particles->m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            ifelse(@D@,2,
                   gzprintf(fp,"%f %f\n",particles->f[i][j][0],
                            particles->f[i][j][1]);,
                   @D@,3,
                   gzprintf(fp,"%f %f %f\n",particles->f[i][j][0],
                            particles->f[i][j][1],particles->f[i][j][2]);)
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stresses@D@_text(char *filename,soft_particles@D@ *particles,
                            box_geometry *geometry)
{
    FILE *fp;
    symmetric_tensor@D@ sigma;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    calculate_system_stress_tensor@D@(particles,geometry,sigma);

    fp = fopen(filename,"wt");

    ifelse(@D@,2,
           fprintf(fp,"%f %f %f\n",sigma[0],sigma[1],sigma[2]);,
           @D@,3,
           fprintf(fp,"%f %f %f %f %f %f\n",sigma[0],sigma[1],
                   sigma[2],sigma[3],sigma[4],sigma[5]);)

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void calculate_system_stress_tensor@D@(soft_particles@D@ *particles,
                                       box_geometry *geometry,
                                       symmetric_tensor@D@ sigma)
{
    int i,j;
    double v;

    SYMMETRIC_TENSOR@D@(sigma[@@] = 0;)

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            SYMMETRIC_TENSOR@D@(sigma[@@] += particles->sigma[i][j][@@];)
        }
    }

    v = calculate_volume_of_box(geometry);

    SYMMETRIC_TENSOR@D@(sigma[@@] /= v;)
}'`')

/*---------------------------------------------------------------------------*/
