/*---------------------------------------------------------------------------*/

/*!  \file  zget.c
     \brief  some utilities for reading gzipped file.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005-2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "zget.h"

/*---------------------------------------------------------------------------*/
int gzgetstring(gzFile *fp,char *buf,int maxsize)
{
    int i;
    char c;
    int d;

        /*  Skip white space and newline.  */
    while((d = gzgetc(fp)) != EOF)
    {
        c = (char)d;
        if(!((c == ' ') ||
             (c == '\t') ||
             (c == '\n') ||
             (c == 0x0a) ||
             (c == 0x0d)))
            break;
    }

        /*  Read string until white space or newline or EOF.  */
    i = 0;
    do
    {
        c = (char)d;
        if((c == ' ') ||
           (c == '\t') ||
           (c == '\n') ||
           (c == 0x0a) ||
           (c == 0x0d))
        {
            buf[i] = '\0';
            return i;
        }

        buf[i++] = c;

        if(i == maxsize)
            return maxsize;
    } while((d = gzgetc(fp)) != EOF);

    return i;
}

/*---------------------------------------------------------------------------*/
int gzgetdouble(gzFile *fp,double *d)
{
    int i;
    static char buf[ZGET_BUFFER_MAXSIZE];

    i = gzgetstring(fp,buf,ZGET_BUFFER_MAXSIZE);
    if(i == ZGET_BUFFER_MAXSIZE)
    {
        fprintf(stderr,"cannot read data (gzipped string is too long).\n");
        exit(EXIT_FAILURE);
    }

    if(i == 0)
        return 0;

    return sscanf(buf,"%lg",d);
}

/*---------------------------------------------------------------------------*/
int gzgetint(gzFile *fp,int *i)
{
    int j;
    static char buf[ZGET_BUFFER_MAXSIZE];

    j = gzgetstring(fp,buf,ZGET_BUFFER_MAXSIZE);
    if(j == ZGET_BUFFER_MAXSIZE)
    {
        fprintf(stderr,"cannot read data (gzipped string is too long).\n");
        exit(EXIT_FAILURE);
    }

    if(j == 0)
        return 0;

    return sscanf(buf,"%d",i);
}

/*---------------------------------------------------------------------------*/
