/*---------------------------------------------------------------------------*/

/*!  \file  relaxation.h
     \brief  calculate the stress relaxation function.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_RELAXATION_H
#define INCLUDE_RELAXATION_H

/*---------------------------------------------------------------------------*/
/*!  \brief  calculate the stress relaxation moduli from the stress tensor.
     \param  sigma  stress tensor
     \param  dimension  dimension of the system
     \param  volume  volume of the system
     \param  n  number of soft particles
     \param  m  number of files  */
double **calculate_stress_relaxation_moduli(double **sigma,int dimension,
                                            double volume,int m);
/*!  \brief  write the \a stress relaxation modulus into \a filename.
     \param  filename  name of the output file
     \param  g  stress relaxation moduli
     \param  dimension  dimension of the system
     \param  m  number of files
     \param  dt  size of the time step  */
void write_stress_relaxation_moduli_text(char *filename,double **g,
                                         int dimension,int m,double dt);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
