/*---------------------------------------------------------------------------*/

/*!  \file  textread.c
     \brief  read the position and force data from gzipped ascii files.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "xmalloc.h"
#include "textread.h"
#include "zget.h"

#define STRING_BUFFERSIZE 0x100

/*---------------------------------------------------------------------------*/
void read_stress_text_sequential(char *template,double ***sigma,int dimension,
                                int m)
{
    FILE *fp;
    int i,j;
    static char filename[FILENAME_BUFFERSIZE];
    int n_elements;

    if((template == NULL) || (strlen(template) == 0))
        return;

    n_elements = (dimension * (dimension + 1)) / 2;

    *sigma = xmalloc(sizeof(double *) * m);
    **sigma = xmalloc(sizeof(double) * m * n_elements);
    for(i = 1;i < m;i++)
    {
        (*sigma)[i] = (*sigma)[i - 1] + n_elements;
    }

    for(i = 0;i < m;i++)
    {
        sprintf(filename,template,i);

        fp = fopen(filename,"rb");

        for(j = 0;j < n_elements;j++)
        {
            fscanf(fp,"%lf",&((*sigma)[i][j]));
        }

        fclose(fp);
    }
}

/*---------------------------------------------------------------------------*/
void read_geometry_text(char *filename,int *dimension,double *l,
                        double *delta_l)
{
    FILE *fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = fopen(filename,"r");

        /*  dimension  */
    fscanf(fp,"%d",dimension);
    if(((*dimension) != 2) && ((*dimension) != 3))
    {
        fclose(fp);

        fprintf(stderr,"cannot read data from a file `%s' (incorrect format)\n",filename);
        exit(EXIT_FAILURE);
    }

        /*  edge lengthes  */
    for(i = 0;i < (*dimension);i++)
    {
        fscanf(fp,"%lf",&(l[i]));
    }

        /*  gap  */
    fscanf(fp,"%lf",delta_l);
}

/*---------------------------------------------------------------------------*/
void read_positions_text(char *filename,double ***r,int *dimension,
                         int *n,int **m)
{
    gzFile *fp;
    int i,j,k;
    static char buf[STRING_BUFFERSIZE];
    int m_tot;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"rb");

        /*  check the header  */
    gzgetstring(fp,buf,STRING_BUFFERSIZE);
    if(strcmp(buf,"#") != 0)
    {
        goto error;
    }

    gzgetint(fp,dimension);

    gzgetint(fp,n);

    gzgetstring(fp,buf,STRING_BUFFERSIZE);
    if(strcmp(buf,"#") != 0)
    {
        goto error;
    }

    (*m) = xmalloc(sizeof(int) * (*n));
    m_tot = 0;
    for(i = 0;i < (*n);i++)
    {
        gzgetint(fp,&((*m)[i]));
        m_tot += (*m)[i];
    }


    *r =xmalloc(sizeof(double *) * (*n));
    **r =xmalloc(sizeof(double) * m_tot * (*dimension));
    for(i = 1;i < (*n);i++)
    {
        (*r)[i] = (*r)[i - 1] + ((*m)[i - 1]) * (*dimension);
    }

    for(i = 0;i < (*n);i++)
    {
        for(j = 0;j < (*m)[i];j++)
        {
            for(k = 0;k < (*dimension);k++)
            {
                gzgetdouble(fp,&((*r)[i][j * (*dimension) + k]));
            }
        }
    }

    gzclose(fp);
    return;


  error:
    gzclose(fp);

    fprintf(stderr,"cannot read data from a file `%s' (incorrect format or size mismatch)\n",filename);
    exit(EXIT_FAILURE);
}


/*---------------------------------------------------------------------------*/
