/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.h
     \brief  output the position data as gzipped ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_TEXTWRITE_H
#define INCLUDE_TEXTWRITE_H

#include "particles.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  write positions of particles to \a filename.
     \param  filename  file name
     \param  r  positions of soft particles
     \param  dimension  dimension of the system
     \param  n  number of particle species
     \param  m  numbers of soft particles  */
void write_positions_text(char *filename,double **r,int dimension,
                          int n,int *m);
/*!  \brief  write positions of particles to \a filename.
     \param  filename  file name
     \param  dimension  dimension of the system
     \param  l  edge lengthes for the simulation box
     \param  delta_l  gap between cells by shear deformation  */
void write_geometry_text(char *filename,int dimension,double *l,
                         double delta_l);
/*!  \brief  write bond vectors to \a filename.
     \param  filename  file name
     \param  u  bond vectors
     \param  dimension  dimension of the system
     \param  m  numbers of soft dumbbells  */
void write_bond_text(char *filename,double *u,int dimension,int m);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
